'''Tunnel to use to connect to other nodes over Tor.'''
from libs import socks
import socket
import libs.threadmanager
from libs.events import broadcast

class Tunnel(object):
    '''Base Tunnel class. Should be subclassed by other tunnels for
    compatibility reasons as well as ease of use.
    '''
    def __init__(self, node):
        self.node = node
        self.is_connected = False
    
    def connect(self):
        '''Use this tunnel to connect to `address`:`port`'''
        self.connection = Connection(self.node)
        success = self.connection.connect()
        if success:
            self.is_connected = True
        else:
            self.is_connected = False
        return True
    
    def connected(self):
        return self.is_connected

    def disconnect(self, nodeid):
        '''Disconnect from a node at `nodeid`'''
        pass
        
    def send(self, message):
        if self.connected:
            self.connection.send(message)
        else:
            self.connected()
            
        if self.connected:
            self.connection.send(message)
        else:
            return False

class Connection(object):
    '''A class to store a connection to a peer'''
    def __init__(self, node):
        # node.ip must not be unicode because of this bug in python-socks:
        #   https://code.google.com/p/httplib2/issues/detail?id=179
        self.node = (str(node.ip), node.port)
        self.sock = socks.socksocket()
        self.sock.setproxy(socks.PROXY_TYPE_SOCKS5, addr='127.0.0.1',
                           port=9050, rdns=True)
        
    def connect(self):        
        try:
            self.sock.connect(self.node)
            
        except socks.Socks5Error as error:
            if error.errno == 2: # not connected
                return False
            
        return True
        
    def send(self, message):
        broadcast('logthis', 'Sending: ' + message, level = 0)
        self.sock.send(message)

class Listener(libs.threadmanager.Thread):
    '''A class to listen for connections sent to our Tor onion address.
    Messages to our Tor onion address should be recieved on a localhost
    port unless otherwise set. We have no way to associate which friend
    this is so it must be added to the handshake somehow.
    '''
    def __init__(self, port=4040):
        super(Listener, self).__init__()
        self.sock = socket.socket()
        self.sock.bind(('localhost', port))
        self.sock.listen(2)

    def run(self):
        while not self._stop.isSet():
            # TODO: find out which friend this is without IP address
            connection = self.sock.accept() # TODO: blocks shutdown
            print('----------------------------------')
            print('Got a connection on the Tor tunnel')
            # TODO: convert this connection to a Connection object!!
            # TODO: send this connection to libs.friends

def start():
    listener = Listener()
    listener.start()
    libs.threadmanager.register(listener)